
#ifdef XSCREENSAVER

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <SDL/SDL.h>
#include <X11/Xlib.h>

#include "aquarium.h"
#include "fish.h"
#include "bubble.h"
#include "vroot.h"


static int screen_width;
static int screen_height;
static int depth=24;


static SDL_Surface *screen=NULL;
static SDL_Surface *screen_image, *background;
static SDL_Rect *fish_dest, *fish_src, *clean_dest;

static int curr_dest, clean_count;

static char *sdl_command=NULL;
static int keep_resolution, escape_only;



/* This is not nice!! */
static SDL_Surface *thisfish[MAX_FISHES];


void xscreensaver_draw_image(int x, int y, int idx, int rev, SA_Image *image)
{

    fish_dest[curr_dest].x=x;
    fish_dest[curr_dest].y=y;
    fish_dest[curr_dest].w=image->width;
    fish_dest[curr_dest].h=image->height;

    fish_src[curr_dest].x=0;
    fish_src[curr_dest].y=(int)((float)idx*(float)image->full_height/(float)image->frames+0.5);
    fish_src[curr_dest].w=image->width;
    fish_src[curr_dest].h=image->height;

    if(!rev)
	thisfish[curr_dest]=SDL_CreateRGBSurfaceFrom(image->image,
						     image->width,
						     (int)((float)image->full_height*(float)(idx) / 
							   (float)image->frames + 0.5),
						     32,image->rowstride,
						     0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000);
    else
	thisfish[curr_dest]=SDL_CreateRGBSurfaceFrom(image->rev,
						     image->width,
						     (int)((float)image->full_height*(float)(idx+1) / 
							   (float)image->frames),
						     32,image->rowstride,
						     0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000);

    curr_dest++;

    if(curr_dest>MAX_FISHES){
	printf("Waring!!! Too many fishes - Recompile and increase the #define MAX_FISHES!!\n"); 
	exit(-1);
    }

}




void xscreensaver_clean(int x,int y,int w,int h)
{

    clean_dest[clean_count].x=x-5;
    clean_dest[clean_count].y=y-5;
    clean_dest[clean_count].w=w+10;
    clean_dest[clean_count].h=h+10;


    SDL_BlitSurface(background,&clean_dest[clean_count],screen,&clean_dest[clean_count]);

    clean_count++;
    if(clean_count>MAX_FISHES){
	printf("Waring!!! Too many fishes - Recompile and increase the #define MAX_FISHES!!\n"); 
	exit(-1);
    }

}

void xscreensaver_quit()
{

    g_free(sdl_command);

    /* Resetting the term signal to the orignal so we can quit nicely.*/
    signal(SIGTERM, SIG_DFL);
    /* Die! */
    kill(getpid(),SIGTERM);
}


void xscreensaver_main(void)
{
    SDL_Rect dest;
    SDL_Event event;
    AquariumData *ad;

    Fish_settings *fish_settings;
    Bubble_settings *bubble_settings;

    clock_t totaltime1, totaltime2;
    clock_t cali1, cali2;


    int num_events=0;
    int main_loop=0;
    int totalframes=0, frames=0;
    int i;
    int delay=0;

    ad = aquarium_get_settings_ptr();
    fish_settings = fish_get_settings_ptr();
    bubble_settings = bubble_get_settings_ptr();

    if ( SDL_Init(SDL_INIT_VIDEO) < 0 ){
	printf("Can't init SDL: %s\n",SDL_GetError());
	exit(1);
    }


    /* So things are resorted to normal if sherman's get terminated - Like by xscreensaver*/
    if(ad->xscreensaver!=0)
	signal(SIGTERM,xscreensaver_quit);
    else
	atexit(SDL_Quit);




    /* I don't pass any other flags than xscreensaver, because then it will
       hopefully work on nearly every graphic card. */

    if(ad->xscreensaver==0){
	if(!SDL_VideoModeOK(screen_width,screen_height,depth, SDL_FULLSCREEN)){
	    printf("Sorry, video mode %dx%d in %d bits isn't supported by hardware\n",
		   screen_width,screen_height,depth);
	    exit(2);
	}
    }


    if(ad->xscreensaver!=0){
	screen=SDL_SetVideoMode(screen_width,screen_height,0, SDL_NOFRAME);
    }
    else{
	if(ad->keep_resolution)
	    screen=SDL_SetVideoMode(screen_width,screen_height,0, SDL_NOFRAME);
	else
	    screen=SDL_SetVideoMode(screen_width,screen_height,depth, SDL_FULLSCREEN);
    }


    if(screen == NULL){
	printf("Unable to set video mode %dx%d in %d bits.\n",
	       screen_width,screen_height,depth);
	exit(3);

    }

    /* Hide the mouse cursor */
    SDL_ShowCursor(0);

    /* Start with all black */
    SDL_FillRect(screen,NULL,0x000000);



    fish_turn();

    fish_dest=g_malloc0(sizeof(SDL_Rect)*(fish_settings->num_fish+bubble_settings->max_bubbles_fullscreen));
    fish_src=g_malloc0(sizeof(SDL_Rect)*(fish_settings->num_fish+bubble_settings->max_bubbles_fullscreen));
    clean_dest=g_malloc0(sizeof(SDL_Rect)*(fish_settings->num_fish+bubble_settings->max_bubbles_fullscreen));

    dest.x=0;
    dest.y=0;
    dest.w=ad->xmax;
    dest.h=ad->ymax;

    screen_image=SDL_CreateRGBSurfaceFrom(ad->rgb,ad->xmax,ad->ymax,depth,ad->xmax*3, 0x0000ff, 0x00ff00, 0xff0000, 0);
    background=SDL_CreateRGBSurfaceFrom(ad->bgr,ad->xmax,ad->ymax,depth,ad->xmax*3, 0x0000ff, 0x00ff00, 0xff0000, 0);



    /*
       Wait for 1/4 second for Xscreensaver to start up nicely. 
       Sometimes needed on slow machines (Like my Pentium II 300 MHz)
    */
    if(ad->xscreensaver!=0)
	usleep(250000);

    SDL_BlitSurface(screen_image,&dest,screen,&dest);

    SDL_UpdateRect(screen,0,0,0,0);

    totaltime1=clock();

    cali1=clock();

    while(!main_loop){

	curr_dest=0;
	clean_count=0;

	aquarium_update(NULL);

	for(i=0;i<curr_dest;i++){
	    SDL_BlitSurface(thisfish[i],&fish_src[i],screen,&fish_dest[i]);
	}


	/* If we get a SIGTERM from xscreensaver in this loop, and we later do a SDL_Quit()
	   X will get problems. So we have to avoid calling SDL_Quit if a sigterm is caught
	   in this loop.

	   Otherwise we get:
	   Xlib: unexpected async reply (sequence 0xf03)!
	   And sherman's starts eating processor power like hell!
	*/

	for(i=0;i<curr_dest;i++){
	    SDL_UpdateRects(screen,1,&clean_dest[i]);
	    SDL_FreeSurface(thisfish[i]);
	}

	totalframes++;
	frames++;

	/* trying to calibrate so no matter what fishes that are displayed and on
	   all machines, the number of frames should be max 50 per second. */

	/* After ten frames, 1/5 second should have passed by */

	/* But I doubt this piece of code is working at all! */

	if(frames==10){
	    cali2=clock();

	    /* Check if we're going too fast! */
	    if((float)(cali2-cali1) < (float)(0.2*CLOCKS_PER_SEC)){
		delay=(int)(((float)((0.2*CLOCKS_PER_SEC)-(cali2-cali1))/CLOCKS_PER_SEC)*100000);
	    }
	    else{
		if(delay!=0){
		    if(delay<((int)(((float)((0.2*CLOCKS_PER_SEC)-(cali2-cali1))/CLOCKS_PER_SEC)*100000))){
			delay-=(int)(((float)((0.2*CLOCKS_PER_SEC)-(cali2-cali1))/CLOCKS_PER_SEC)*100000);
		    }
		    else
			delay=0;
		}
	    }
	    /*      printf("delay:%d\n",delay);*/
	    cali1=clock();
	    frames=0;
	}

	usleep(delay);



	while(SDL_PollEvent(&event)){

	    if(escape_only){
		if(event.type==SDL_KEYDOWN)
		    if(event.key.keysym.sym == SDLK_ESCAPE) main_loop=1;
	    }
	    else{
		switch(event.type){
		case SDL_QUIT:
		    if(ad->xscreensaver==0) main_loop=1;
		    break;
		case SDL_KEYDOWN:
		    num_events++;
		    if(num_events==2)
			main_loop=1;
		    break;
		case SDL_MOUSEMOTION:
		    num_events++;
		    if(num_events==2)
			main_loop=1;
		    break;
		case  SDL_MOUSEBUTTONDOWN:
		    num_events++;
		    if(num_events==2)
			main_loop=1;
		    break;
		}
	    }
	}

    
    }
    totaltime2=clock();
    printf("Frames: %d\n",totalframes);
    printf("Seconds: %f\n",(float)(totaltime2-totaltime1)/CLOCKS_PER_SEC);
    printf("FPS: %f\n",(float)totalframes/((float)(totaltime2-totaltime1)/CLOCKS_PER_SEC));

    exit(0);

}



void xscreensaver_init(int width, int height)
{
    AquariumData *ad;
    Window win=0;

    ad = aquarium_get_settings_ptr();

    screen_height=height;
    screen_width=width;

    ad->xmax=ad->windowsize_x=screen_width;
    ad->ymax=ad->windowsize_x=screen_height;
  
    ad->virtual_aquarium_x = ad->xmax + 2 * VIRTUAL_AQUARIUM_DX;
    ad->virtual_aquarium_y = ad->ymax + 2 * VIRTUAL_AQUARIUM_DY;


    if(ad->xscreensaver!=0){
	if(ad->display==NULL) ad->display=XOpenDisplay(NULL);

	win = RootWindowOfScreen(ScreenOfDisplay(ad->display, DefaultScreen(ad->display)));

	sdl_command=g_strdup_printf("SDL_WINDOWID=%ld",win);
	putenv(sdl_command);

    }


}

void aquarium_fullscreen_update(void)
{
    fish_update();
    bubble_update();

}

int main(int argc, char **argv)
{
    srand(time(NULL)*getpid());

    keep_resolution=0;
    escape_only = 0;

    fish_load_settings_f();

    if(ad.full_screen){
        if(window_id==0){
            if(ad.keep_resolution!=0)
                xscreensaver_init(gdk_screen_width(),gdk_screen_height());
            else
                xscreensaver_init(640,480);
        }
        else{
            ad.full_screen=0;
            /* FIX ME - Xscreensaver window */
        }
    }


    XSCREENSAVER_main();



    return 0;
}

#endif
